#!/usr/bin/python
# -*- coding: utf-8 -*-

from os import path, urandom


class Config(object):
    DEBUG = False
    PORT = 5000
    HOST = '0.0.0.0'
    URL_PREFIX = '/api'
    PROJECT_ROOT = path.abspath(path.dirname(__file__))
    TEMPLATE_FOLDER = path.join(PROJECT_ROOT, 'templates')

    SQLALCHEMY_DATABASE_URI = "sqlite://"
    DATABASE_CONNECT_OPTIONS = {}

    SECURITY_TRACKABLE = True

    BUNDLE_ERRORS = True
    TRAP_HTTP_EXCEPTIONS = True


class Development(Config):
    DEBUG = True
    SECRET_KEY = 'development'
    PROJECT_SALT = 'dev salt'


class Production(Config):
    DEBUG = False
    TESTING = False


    # =======================================================
    # = NB: USING URANDOM INVALIDATES ALL CURRENT SESSIONS! =
    # =======================================================
    # Generate a new random secret key
    SECRET_KEY = urandom(16)
    # =============================================================================================
    # =  NB! DO NOT CHANGE THIS IF YOU HAVE A RUNNING DATABASE - THIS WILL INVALIDATE ALL LOGIN!  =
    # =============================================================================================
    # Simple 64-bit string generated by os.urandom(8)
    PROJECT_SALT = '\xac\xf9\xc6\x96R\xc0\xa1\x05'


class Testing(Config):
    TESTING = True
    SECRET_KEY = 'testing'
    PROJECT_SALT = 'test salt'
